@@ -197,7 +197,7 @@ width=%d, height=%d)",
 	char buffer[22];
 
 	if (zio_lseek(&book->subbook_current->text_zio,
-	    (position->page - 1) * EB_SIZE_PAGE + position->offset,
+	    ((off_t) position->page - 1) * EB_SIZE_PAGE + position->offset,
 	    SEEK_SET) < 0) {
 	    error_code = EB_ERR_FAIL_SEEK_BINARY;
 	    goto failed;
@@ -261,8 +261,8 @@ width=%d, height=%d)",
     context = &book->binary_context;
     context->code = EB_BINARY_MONO_GRAPHIC;
     context->zio = &book->subbook_current->text_zio;
-    context->location = (position->page - 1) * EB_SIZE_PAGE + position->offset
-	+ (width + 7) / 8 * (height - 1);
+    context->location = ((off_t) position->page - 1) * EB_SIZE_PAGE
+	+ position->offset + (width + 7) / 8 * (height - 1);
     context->size = (width + 7) / 8 * height;
     context->offset = 0;
     context->cache_offset = 0;
@@ -450,7 +450,7 @@ width=%d, height=%d)",
 	char buffer[22];
 
 	if (zio_lseek(&book->subbook_current->text_zio,
-	    (position->page - 1) * EB_SIZE_PAGE + position->offset,
+	    ((off_t) position->page - 1) * EB_SIZE_PAGE + position->offset,
 	    SEEK_SET) < 0) {
 	    error_code = EB_ERR_FAIL_SEEK_BINARY;
 	    goto failed;
@@ -509,8 +509,8 @@ width=%d, height=%d)",
 
     context->code = EB_BINARY_GRAY_GRAPHIC;
     context->zio = &book->subbook_current->text_zio;
-    context->location = (position->page - 1) * EB_SIZE_PAGE + position->offset
-	+ (width + 1) / 2 * (height - 1);
+    context->location = ((off_t) position->page - 1) * EB_SIZE_PAGE
+	+ position->offset + (width + 1) / 2 * (height - 1);
     context->size = (width + 1) / 2 * height;
     context->offset = 0;
     context->cache_offset = 0;
@@ -621,9 +621,9 @@ end_position={%d,%d})",
 	goto failed;
     }
 
-    start_location = (start_position->page - 1) * EB_SIZE_PAGE
+    start_location = ((off_t) start_position->page - 1) * EB_SIZE_PAGE
 	+ start_position->offset;
-    end_location   = (end_position->page - 1)   * EB_SIZE_PAGE
+    end_location   = ((off_t) end_position->page - 1)   * EB_SIZE_PAGE
 	+ end_position->offset;
 
     context = &book->binary_context;
@@ -677,8 +677,8 @@ end_position={%d,%d})",
 	    context->size = 0;
     } else {
 	if (zio_lseek(context->zio,
-	    (book->subbook_current->sound.start_page - 1) * EB_SIZE_PAGE + 32,
-	    SEEK_SET) < 0) {
+	    ((off_t) book->subbook_current->sound.start_page - 1)
+		* EB_SIZE_PAGE + 32, SEEK_SET) < 0) {
 	    error_code = EB_ERR_FAIL_SEEK_BINARY;
 	    goto failed;
 	}
@@ -787,7 +787,8 @@ eb_set_binary_color_graphic(EB_Book *book, const EB_Position *position)
     context = &book->binary_context;
     context->code = EB_BINARY_COLOR_GRAPHIC;
     context->zio = &book->subbook_current->graphic_zio;
-    context->location = (position->page - 1) * EB_SIZE_PAGE + position->offset;
+    context->location = ((off_t) position->page - 1) * EB_SIZE_PAGE
+	+ position->offset;
     context->offset = 0;
     context->cache_length = 0;
     context->cache_offset = 0;
@@ -879,7 +880,7 @@ eb_set_binary_mpeg(EB_Book *book, const unsigned int *argv)
     LOG(("aux: eb_set_binary_mpeg(): movie_file_name=%s", movie_file_name));
 
     if (eb_find_file_name3(book->path, subbook->directory_name,
-	subbook->movie_directory_name, movie_file_name, movie_path_name)
+	subbook->movie_directory_name, movie_file_name, movie_file_name)
 	!= EB_SUCCESS) {
 	error_code = EB_ERR_NO_SUCH_BINARY;
 	goto failed;
@@ -1205,7 +1206,7 @@ eb_read_binary_mono_graphic(EB_Book *book, size_t binary_max_length,
 	 */
 	if (context->offset != 0
 	    && context->offset % line_length == 0
-	    && zio_lseek(context->zio, (off_t)line_length * -2, SEEK_CUR)
+	    && zio_lseek(context->zio, (off_t) line_length * -2, SEEK_CUR)
 	    < 0) {
 	    error_code = EB_ERR_FAIL_SEEK_BINARY;
 	    goto failed;
@@ -1336,7 +1337,7 @@ eb_read_binary_gray_graphic(EB_Book *book, size_t binary_max_length,
 	 */
 	if (context->offset != 0
 	    && context->offset % line_length == 0
-	    && zio_lseek(context->zio, (off_t)line_length * -2, SEEK_CUR)
+	    && zio_lseek(context->zio, (off_t) line_length * -2, SEEK_CUR)
 	    < 0) {
 		error_code = EB_ERR_FAIL_SEEK_BINARY;
 		goto failed;
