@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2004  Motoyuki Kasahara
+ * Copyright (c) 2003-2006  Motoyuki Kasahara
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -112,7 +112,7 @@ struct EBNet_Socket_Entry_Struct {
     off_t offset;
 
     /* file size */
-    size_t file_size;
+    off_t file_size;
 };
 
 static EBNet_Socket_Entry *ebnet_socket_entries;
@@ -431,6 +431,14 @@ ebnet_get_addresses(const char *host, char *ipv6_address, char *ipv4_address)
 	    ipv6_address, INET6_ADDRSTRLEN + IF_NAMESIZE,
 	    dummy_service, sizeof(dummy_service),
 	    NI_NUMERICHOST | NI_NUMERICSERV | NI_WITHSCOPEID);
+#if NI_WITHSCOPEID != 0
+	if (gai_error != 0) {
+	    gai_error = getnameinfo(info->ai_addr, info->ai_addrlen,
+		ipv6_address, INET6_ADDRSTRLEN + IF_NAMESIZE,
+		dummy_service, sizeof(dummy_service),
+		NI_NUMERICHOST | NI_NUMERICSERV);
+	}
+#endif
 	if (gai_error == 0)
 	    break;
 	*ipv6_address = '\0';
@@ -833,7 +841,7 @@ ebnet_set_book_name(int file, const char *book_name)
 	return -1;
 
     strncpy(entry->book_name, book_name, EBNET_MAX_BOOK_NAME_LENGTH + 1);
-    *(entry->book_name + EBNET_MAX_BOOK_NAME_LENGTH + 1) = '\0';
+    *(entry->book_name + EBNET_MAX_BOOK_NAME_LENGTH) = '\0';
 
     return 0;
 }
@@ -927,7 +935,7 @@ ebnet_get_offset(int file)
  * Set file size.
  */
 int
-ebnet_set_file_size(int file, size_t file_size)
+ebnet_set_file_size(int file, off_t file_size)
 {
     EBNet_Socket_Entry *entry;
 
@@ -943,7 +951,7 @@ ebnet_set_file_size(int file, size_t file_size)
 /*
  * Get file size.
  */
-ssize_t
+off_t
 ebnet_get_file_size(int file)
 {
     EBNet_Socket_Entry *entry;
