@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2004  Motoyuki Kasahara
+ * Copyright (c) 1997-2006  Motoyuki Kasahara
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -206,6 +206,7 @@ eb_initialize_searches(EB_Book *book)
     eb_initialize_search(&subbook->keyword);
     eb_initialize_search(&subbook->cross);
     eb_initialize_search(&subbook->menu);
+    eb_initialize_search(&subbook->image_menu);
     eb_initialize_search(&subbook->copyright);
     eb_initialize_search(&subbook->text);
     eb_initialize_search(&subbook->sound);
@@ -248,6 +249,7 @@ eb_finalize_searches(EB_Book *book)
     eb_finalize_search(&subbook->endword_kana);
     eb_finalize_search(&subbook->keyword);
     eb_finalize_search(&subbook->menu);
+    eb_finalize_search(&subbook->image_menu);
     eb_finalize_search(&subbook->copyright);
     eb_finalize_search(&subbook->text);
     eb_finalize_search(&subbook->sound);
@@ -299,7 +301,7 @@ eb_presearch_word(EB_Book *book, EB_Search_Context *context)
 	 * Seek and read a page.
 	 */
 	if (zio_lseek(&book->subbook_current->text_zio,
-	    (context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
+	    ((off_t) context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
 	    cache_book_code = EB_BOOK_NONE;
 	    error_code = EB_ERR_FAIL_SEEK_TEXT;
 	    goto failed;
@@ -603,7 +605,7 @@ eb_hit_list_word(EB_Book *book, EB_Search_Context *context, int max_hit_count,
 	 */
 	if (cache_book_code != book->code || cache_page != context->page) {
 	    if (zio_lseek(&book->subbook_current->text_zio,
-		(context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
+		((off_t) context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
 		error_code = EB_ERR_FAIL_SEEK_TEXT;
 		goto failed;
 	    }
@@ -929,7 +931,7 @@ eb_hit_list_keyword(EB_Book *book, EB_Search_Context *context,
 	 */
 	if (cache_book_code != book->code || cache_page != context->page) {
 	    if (zio_lseek(&book->subbook_current->text_zio,
-		(context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
+		((off_t) context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
 		error_code = EB_ERR_FAIL_SEEK_TEXT;
 		goto failed;
 	    }
@@ -1261,7 +1263,7 @@ eb_hit_list_multi(EB_Book *book, EB_Search_Context *context, int max_hit_count,
 	 */
 	if (cache_book_code != book->code || cache_page != context->page) {
 	    if (zio_lseek(&book->subbook_current->text_zio,
-		(context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
+		((off_t) context->page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
 		error_code = EB_ERR_FAIL_SEEK_TEXT;
 		goto failed;
 	    }
@@ -1650,5 +1652,5 @@ eb_and_hit_lists(EB_Hit and_list[EB_TMP_MAX_HITS], int *and_count,
     for (i = 0; i < hit_list_count; i++)
 	hit_counts[i] = hit_indexes[i];
 
-    LOG(("out: eb_and_hit_lists(and_count=%d)", and_count));
+    LOG(("out: eb_and_hit_lists(and_count=%d)", *and_count));
 }
