@@ -1,5 +1,5 @@
 /*                                                            -*- C -*-
- * Copyright (c) 2001-2004  Motoyuki Kasahara
+ * Copyright (c) 2001-2006  Motoyuki Kasahara
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -79,6 +79,16 @@ int strncasecmp(const char *, const char *, size_t);
 #endif /* not MAXPATHLEN */
 #endif /* not PATH_MAX */
 
+/*
+ * rename() on Windows complains if the new file already exists.
+ * We fake rename() here for Windows.
+ */
+#ifdef WIN32
+#include <windows.h>
+#define rename(old, new) \
+    (MoveFileEx((old), (new), MOVEFILE_REPLACE_EXISTING) ? 0 : -1)
+#endif
+
 #include "eb.h"
 #include "error.h"
 #include "build-post.h"
